﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Service
Version=9.801
@EndOfDesignText@
#Region  Service Attributes 
	#StartAtBoot: False
#End Region


' When started the service immediately starts playing tracks from the /Music/ folder.
' These subroutines can be called by the client :
'		register	: provides callback information for player events
'		setRepeat	: sets switch to repeat current track
'		goNext	: starts playing the next track
'		goBack	:	restarts current track; within 2 secs of start, plays previous track, if any
'		goSkip	: replaces next track
'		goTo(perCent)	: change playing position to [perCent] of current track
'		goPause	: toggle player pause
'		setFolder : selects single folder for pool
'		deleteTrack	: deletes the current track from the 'phone

'	Once a caller and callback routine have been set the following callbacks occur :
'		- current track has changed
'		- next track has changed
'		- progress timer at 1 sec intervals

Sub Process_Globals
		
	Type track (filepath As String, title As String, artist As String, album As String, _
							flag As Int)
	
	Type timeStamp (perCent As Float, position As Long, duration As Long)
	
	Dim mp As MediaPlayer
	Dim tmr As Timer													' Initiates time-stamps
	
	Dim pool As List													' Unplayed mp3 files as [track] objects
	Dim played As List												' Stack of played [track] objects
	Dim stacked As List												' Tracks waiting to be next (after back-tracking)
	Dim outpool As List												' Tracks currently out of selection
	
	Dim currentTrack As track									' Track currently being played
	Dim nextTrack As track										' Next track to be played
	Dim repeat As Boolean = False							' Set TRUE to repeat current track
	Dim paused As Boolean											' TRUE while player is paused
	
	Dim client As Object = Null								' Client activity
	Dim callback As String										' Callback sub-routine
	
	Dim NEW_CURRENT As Int = 1								' Flags used in callback events
	Dim NEW_NEXT As Int = 2
	Dim TIME_STAMP As Int = 3
	
	Dim lock As PhoneWakeState
	Dim notify As Bitmap											' Notification icon source
	Dim id As Int = 1													' Notification identifier

End Sub

Private Sub Service_Create
	Service.AutomaticForegroundMode = Service.AUTOMATIC_FOREGROUND_NEVER
	lock.PartialLock
	mp.Initialize2("mp")
	pool.Initialize
	played.Initialize
	stacked.Initialize
	outpool.Initialize
	tmr.Initialize("tmr", 1000)
	tmr.Enabled = True
	notify = LoadBitmapResize(File.DirAssets, "player.png", 24dip, 24dip, False)
End Sub

Private Sub Service_Start (StartingIntent As Intent)
	Service.StartForeground(id, CreateNotification("MP3 Player"))
	startPlayer
End Sub

Private Sub Service_Destroy
	lock.ReleasePartialLock
End Sub

' 
'Sub CreateNotification As Notification
'	Dim n As NB6
'	n.Initialize("default", Application.LabelName, "LOW").AutoCancel(True).SmallIcon(notify)
'	Return n.Build("", "", "tag1", Main)
'End Sub

' 
Private Sub CreateNotification (Body As String) As Notification
	Dim notification As Notification
	notification.Initialize2(notification.IMPORTANCE_LOW)
	notification.Icon = "notify"
'	notification.Vibrate = False
	notification.Sound = False
	notification.SetInfo("MP3 Player", Body, Main)
	Return notification
End Sub

Public Sub stop
	Service.StopForeground(id)
	StopService(Me)
End Sub

' Build the pool list of tracks and setup the current and next tracks
Private Sub startPlayer
'	findFiles(File.DirRootExternal & "/Music", "mp3", pool)
  Dim i As Int = 1
	For i = 1 To 5
		Dim t As track
		t.Initialize
		t.filepath = "music/" & Chr(96 + i) & ".mp3"
		pool.Add(t)
	Next
	prepareNextTrack
	playNextTrack
End Sub

' Register the calling client and pass back current track status
Public Sub register(caller As Object, handler As String)
	client = caller
	callback = handler
	CallSub3(client, callback, NEW_CURRENT, currentTrack)
	CallSub3(client, callback, NEW_NEXT, nextTrack)
End Sub

' Call client with message denoted by [flag]
Private Sub callClient(flag As Int)
	If (client = Null) Then Return						' Ignore if no contact details
	If (flag = TIME_STAMP) Then								' Build and send time-stamp
		Dim ts As timeStamp
		ts.perCent = mp.Position / mp.Duration
		ts.position = mp.Position
		ts.duration = mp.Duration
		CallSub3(client, callback, flag, ts)
	Else																			' Build and send track status
		Dim tr As track
		If (flag = NEW_CURRENT) Then tr = currentTrack
		If (flag = NEW_NEXT) Then tr = nextTrack
		CallSub3(client, callback, flag, tr)
	End If
End Sub

' Handle tick event by sending time-stamp to client
Private Sub tmr_Tick
	callClient(TIME_STAMP)	
End Sub

' At end-of-track start next track unless [repeat] is set
Private Sub mp_Complete
	If repeat Then														' Repeat current track
		playCurrentTrack
		repeat = False
	Else
		played.Add(currentTrack)								' Save [currentTrack] on stack
		playNextTrack														' Move to next track
	End If	
End Sub

' Called by client to stop current track and play next track
Public Sub goNext
	If Not(currentTrack = Null) Then played.Add(currentTrack)
	playNextTrack															' Start playing new track
End Sub

' Called by client to replay current track, or previous track
Public Sub goBack
	' If less than 3 secs into track play prior track if there is one
	If (mp.Position < 3000) And (played.Size > 0) Then
		stacked.Add(nextTrack)									' Push next queued track on to [stacked]
		nextTrack = currentTrack								' Move current track back to [Next]
		callClient(NEW_NEXT)
		currentTrack = getFromStack(played)			' Get most recently played track
		callClient(NEW_CURRENT)
		playCurrentTrack 
	Else
		mp.Position = 0													' Return to start of current track
	End If
End Sub

' Called by client to replace the next track with another choice
Public Sub goSkip
	prepareNextTrack
End Sub

' Toggle player pause state
Public Sub goPause
	If paused Then mp.Play Else mp.Pause
	paused = Not(paused)
End Sub

' Called by client to repeat current track when finished
Public Sub setRepeat
	repeat = True
End Sub

' Go to specified position in current track
Public Sub goTo(perCent As Float)
	mp.Position = mp.Duration * perCent
End Sub

' Set folder as new [pool]
Public Sub setFolder(folder As String)
	outpool.AddAll(played)										' Move all tracks into outpool]
	played.Clear
	outpool.AddAll(pool)
	pool.Clear
	For Each t As track In outpool
		If t.filepath.Contains(folder) Then pool.Add(t)
	Next
	If (pool.Size > 2) Then 									' Must be at least three tracks for looping
		prepareNextTrack
		playNextTrack
	Else																			' Reset [pool] to entire list
		pool.AddAll(outpool)
		outpool.Clear
	End If
End Sub

' Permanently delete the current track
Public Sub deleteTrack
	Dim temp As track = currentTrack					' Remember current track details
	currentTrack = Null												' Avoid saving to [played]
	playNextTrack															' Move on to next track
	File.Delete(File.DirRootExternal, temp.filepath)
End Sub

' Start playing [nextTrack] as new [currentTrack]
Private Sub playNextTrack
	currentTrack = nextTrack									' Get next track to play
	callClient(NEW_CURRENT)
	playCurrentTrack
	callClient(TIME_STAMP)
	prepareNextTrack													' Replace next track
End Sub

' Play [currentTrack] in the player
Private Sub playCurrentTrack
'	mp.Load(File.DirRootExternal, currentTrack.filepath)
	mp.Load(File.DirAssets, currentTrack.filepath)
	mp.Play
End Sub

' Get next track from pool, or from [stacked] if not empty
Private Sub prepareNextTrack
	nextTrack = getFromStack(stacked)					' Check for possible stacked track
	If (nextTrack = Null) Then								' Stack was empty
		getNextTrack														' Get track from pool
		getID3Data(nextTrack)										' Retrieve track data
	End If			
	CallSub3(client, callback, NEW_NEXT, nextTrack)
	callClient(NEW_NEXT)											' Keep client updated
End Sub

' Get a new [nextTrack] from the pool
Private Sub getNextTrack
	Dim k As Int = 0													' Random index unless pool nearly empty
	If (pool.Size > 3) Then k = Rnd(0, pool.Size)
	nextTrack = pool.Get(k)										' Get track from pool
	pool.RemoveAt(k)
	If (pool.Size = 0) Then										' If [pool] is empty refill from [played]
		pool.AddAll(played)
		played.Clear
	End If
End Sub

' Pop most recently added track from [stack]
Private Sub getFromStack(stack As List) As track
	Dim result As track = Null
	If (stack.Size > 0) Then									' Get last-added track from [stack]
		result = stack.Get(stack.Size - 1)
		stack.RemoveAt(stack.Size - 1)
	End If
	Return result
End Sub

' Find all files in [path] with [extn] and return as [track] objects in [result]
Private Sub findFiles(path As String, extn As String, data As List)
	Dim files As List = File.ListFiles(path)
	If (files.IsInitialized) Then
		For Each f As String In files
			If (File.IsDirectory(path, f)) Then
				findFiles(path & "/" & f, extn, data)
			Else
				If (f.EndsWith(extn)) Then
					Dim t As track										' Add track data to [data]
					t.Initialize
					t.filepath = path.Replace(File.DirRootExternal, "") & "/" & f
					data.Add(t)
				End If
			End If
		Next
	End If
End Sub

' Retrieve track description from ID3 tag for track
Private Sub getID3Data(trk As track)
'	Dim id3Data As Map = id3Extractor.getID3(File.DirRootExternal, trk.filepath)
'	If id3Data.ContainsKey("TIT2") Then trk.title = id3Data.Get("TIT2")
'	If id3Data.ContainsKey("TPE1") Then trk.artist = id3Data.Get("TPE1")
'	If id3Data.ContainsKey("TALB") Then trk.album = id3Data.Get("TALB")
End Sub
