﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=8.1
@EndOfDesignText@
'	Extracts several forms of ID3 Tag information

'	Shorter than 4096 bytes		: Standard tags - no bitmap
' 4096 bytes long						: non-conformal; padded to 4096 bytes with zeroes
'	Longer than 4096 bytes		: Audiograbber style; often contain bitmaps

' Output is a Map of all text frames

Sub Process_Globals
	
		
End Sub

' Read and decode ID3v2 tag in mp3 file
Public Sub getID3(path As String, filename As String) As Map
	Dim result As Map														' Prepare output map
	result.Initialize
	result.Put("path", path)										' Add file references
	result.Put("file", filename)
	
	Dim frameName As String = "="
	Dim frameSize As Int
	Dim frameContent As String
	Dim encoding As String
	
	Dim buff(1024) As Byte											' Work buffer
	Log(filename)
	File.Copy(File.DirAssets, filename, File.DirInternal, filename)
	Dim raf As RandomAccessFile
	raf.Initialize(path, filename, True)
	raf.ReadBytes(buff, 0, 10, 0)								' Read tag header
	Dim tag As String = BytesToString(buff, 0, 3, "UTF-8")
	' If file does not begin with an ID3 tag then return
	If Not(tag.EqualsIgnoreCase("ID3")) Then Return result
	Dim size As Int
	size = safeSync(buff, 6) + 10								' Calculate total tag length
	
	Dim p As Int = 10														' Prepare to scan tag content
	Do While (p < size)
		raf.ReadBytes(buff, 0, 128, p)
		' If first byte is empty then end of data in 4096 byte tag has been reached
		If buff(0) = 0 Then Return result
		frameName = BytesToString(buff, 0, 4, "UTF-8")
		frameSize = safeSync(buff, 4) + 10
		' Header is followed by one byte encoding flag
		encoding = getEncoding(buff(10))
		' Retrieve frame content if frame contains text
		If frameName.StartsWith("T") Then					' Add only text frame data to [result]
			frameContent = BytesToString(buff, 11, frameSize - 11, encoding)
			result.Put(frameName, frameContent)
'			Log("Frame = " & frameName & "; Size = " & frameSize & "; = " & frameContent)
		Else																			' Process non-text frame
			frameSize = raf.ReadInt(p + 4) + 10
			If frameName.EqualsIgnoreCase("APIC") Then getAPICFrame(buff, p, frameSize, result)
		End If
		p = p + frameSize
	Loop
	raf.Close
	Return result
End Sub

' Add image details (mimetype;start;length) in APIC frame to [info]
Private Sub getAPICFrame(buff() As Byte, offset As Int, framesize As Int, info As Map)
	Dim imageInfo As String										' Image details - mimetype;start;length
	Dim p As Int = 11													' Scan buffer from end of header
	imageInfo = getZeroString(buff, p)				' Get mime description
	p = p + imageInfo.length + 2							' Skip over mime field and picture type
	p = skipToNextZero(buff, p)								' Ignore image description, if any
	p = p + 1
	If (buff(p) = -1) Then										' This byte should be 0xFF - start of image
		' Complete image info with start position in file and image length
		imageInfo = imageInfo & ";"	& (offset + p) & ";" & (framesize - p - 10)
		info.Put("APIC", imageInfo)
	Else
		imageInfo = imageInfo & ";0;0"					' Indicate image data not found 	
	End If
End Sub

' Retrieve bitmap of cover art, if it exists
Public Sub GetBitmap (data As Map) As Bitmap
	Dim result As Bitmap
  Dim raf As RandomAccessFile
	Dim start, length As Int
	
	If data.ContainsKey("APIC") = False Then Return Null
	Dim info As String = data.Get("APIC")
	start = info.SubString2(info.IndexOf(";") + 1, info.LastIndexOf(";"))
	length = info.SubString(info.LastIndexOf(";") + 1)
	If (length < 1) Then Return Null
	
	Dim path As String = data.Get("path")
	Dim filename As String = data.Get("file")
	'Read Image From File into byte buffer
	Dim raf As RandomAccessFile
	raf.Initialize(path, filename, True)
	Dim buff(length) As Byte
	raf.ReadBytes(buff, 0, length, start)
	raf.Close	
	Log(buff(0) & " : " & length)
	
	Dim ips As InputStream
	ips.InitializeFromBytesArray(buff, 0, length + 1)
	result.Initialize2(ips)
	Return result
End Sub

Private Sub getEncoding(n As Int) As String
	If (n = 0) Then Return "8859-1"
	If (n = 1) Then Return "UTF-16"
	If (n = 2) Then Return "UTF16-BE"
	If (n = 3) Then Return "UTF-8"
	Return "UTF-8"
End Sub

' Extract 7-bit encoded hex value, starting at [offset] in [buff]
Private Sub safeSync(buff() As Byte, offset As Int) As Int
	Dim result As Int = 0
	Dim i, n As Int
	Dim b As Byte
	Dim mask As Byte = 0x7F
	For i = 0 To 3
		b = buff(offset + i)
		n = Bit.And(b, mask)
		result = Bit.ShiftLeft(result, 7) + n
	Next
	Return result
End Sub

' Return zero-terminated string starting at [p] in [buff]
Private Sub getZeroString(buff() As Byte, p As Int) As String
	Dim result As String
	Dim i As Int = p
	Do While Not(buff(i) = 0)
		 i = i + 1
	Loop
	result = BytesToString(buff, p, i - p, "UTF-8")
	Return result.trim
End Sub

' Skip through [buff] until next zero byte
Private Sub skipToNextZero(buff() As Byte, p As Int) As Int
	Do While Not(buff(p) = 0)
		p = p + 1
	Loop
	Return p
End Sub

